==> heptapod-rails-0.32.1.tar.gz: heptapod-rails-0.32.1/spec/lib/gitlab/mercurial/hgitaly_commit_spec.rb <==
# frozen_string_literal: true

require "spec_helper"

# rubocop:disable RSpec/FilePath
RSpec.describe Gitlab::Git::Commit, :seed_helper do
  include Gitlab::EncodingHelper
  include RepoHelpers
  include Gitlab::Popen

  def hg_commit(repo, file, date: nil, par: nil, branch: nil)
    date = '0 0' if date.nil?
    repo.hg_call(['up', par]) unless par.nil?

    unless branch.nil?
      repo.hg_call(['branch', "#{branch}"])
    end

    repo.write_file(repo.hg_full_path + "/#{file}",
                       "I am a line in file '#{file}'\n")
    repo.hg_call(['add', "#{file}"])
    repo.hg_call(['ci', '-m', "Commit #{file}", '--config',
                  'experimental.single-head-per-branch=False',
                  '-d', "#{date}"],
                 user: user, for_write: true)
    repo.hg_git_invalidate_maps!
    repo.hgsha_from_rev('.', repository_path)
  end

  before do
    stub_const('HGITALY_TEST_REPO_PATH', 'hgitaly-repo.git')
  end

  let(:repository) do
    repo = Gitlab::Mercurial::HgitalyRepository.new('default', HGITALY_TEST_REPO_PATH, '', 'group/project')
    repo.unit_tests_skip_hooks = true
    repo
  end

  # Same repository as :repository, seen as HgGitRepository, so that
  #
  #   1. we can use HgGitRepository as a reference for assertions on method
  #      results
  #   2. we have easy fixture preparation options
  let(:hg_git_repository) do
    repo = Gitlab::Mercurial::HgGitRepository.new('default', HGITALY_TEST_REPO_PATH, '', 'group/project')
    repo.unit_tests_skip_hooks = true
    repo
  end

  let(:repository_path) { File.join(TestEnv.repos_path, repository.relative_path).chomp('.git') + '.hg' }
  let(:user) { build(:user) }
  let(:non_empty_repository) do
    status = hg_git_repository.create_repository(namespace_fs_path: "group",
                                    in_subgroup: false)
    expect(status).to be(true)
    hg_commit(hg_git_repository, 'foo', date: "0 0")
    repository # HgitalyRepository
  end

  let(:non_empty_repository_path) { repository_path }

  describe '#find_all' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    def hgsha(rev)
      subject.hgsha_from_rev(rev)
    end

    def gitsha(sha)
      hg_git_repository.hg_git_invalidate_maps!
      hg_git_repository.sha_from_hgsha(sha)
    end

    # repo structure:
    #
    #    o    changeset:   2: Commit bar
    #    |
    #    | o  changeset:   1: Commit car (branch: stable)
    #    |/
    #    o  changeset:   0: Commit foo
    #
    let!(:offset) { 86400 } # seconds in 24 hours
    let(:prepare_repo) do
      hg_commit(subject, 'car', par: '0', branch: 'stable',
                date: "#{offset} 0")
      hg_commit(subject, 'bar', par: '0', date: "#{offset * 2} 0")
    end

    let(:hgsha_0) { hgsha('0') }
    let(:hgsha_1) { hgsha('1') }
    let(:hgsha_2) { hgsha('2') }
    let(:gitsha_0) { gitsha(hgsha_0) }
    let(:gitsha_1) { gitsha(hgsha_1) }
    let(:gitsha_2) { gitsha(hgsha_2) }

    it 'returns a collection of commits' do
      commits = described_class.find_all(subject).map(&:id)
      expect(commits).to eq([hgsha_2, hgsha_1, hgsha_0])
      # consistency with HgGitRepository
      commits = described_class.find_all(hg_git_repository).map(&:id)
      expect(commits).to eq([gitsha_2, gitsha_1, gitsha_0])
    end

    context 'max_count' do
      it 'return maximum max_count number of commits' do
        commits = described_class.find_all(subject, max_count: 2).map(&:id)
        expect(commits).to eq([hgsha_2, hgsha_1])
        # consistency with HgGitRepository
        commits = described_class.find_all(hg_git_repository, max_count: 2).map(&:id)
        expect(commits).to eq([gitsha_2, gitsha_1])
      end
    end

    context 'ref + max_count + skip' do
      it 'returns a collection of commits' do
        commits = described_class.find_all(
          subject,
          ref: 'branch/default',
          max_count: 1,
          skip: 1
        ).map(&:id)
        expect(commits).to eq([hgsha_0])
        # consistency with HgGitRepository
        commits = described_class.find_all(
          hg_git_repository,
          ref: 'branch/default',
          max_count: 1,
          skip: 1
        ).map(&:id)
        expect(commits).to eq([gitsha_0])
      end
    end
  end

  describe '#get_messages' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    def hgsha(rev)
      subject.hgsha_from_rev(rev)
    end

    def gitsha(sha)
      hg_git_repository.hg_git_invalidate_maps!
      hg_git_repository.sha_from_hgsha(sha)
    end

    # repo structure:
    #
    #    o    changeset:   2: Commit bar
    #    |
    #    | o  changeset:   1: Commit car (branch: stable)
    #    |/
    #    o  changeset:   0: Commit foo
    #
    let!(:offset) { 86400 } # seconds in 24 hours
    let(:prepare_repo) do
      hg_commit(subject, 'car', par: '0', branch: 'stable',
                date: "#{offset} 0")
      hg_commit(subject, 'bar', par: '0', date: "#{offset * 2} 0")
    end

    let(:hgsha_0) { hgsha('0') }
    let(:hgsha_1) { hgsha('1') }
    let(:hgsha_2) { hgsha('2') }
    let(:gitsha_0) { gitsha(hgsha_0) }
    let(:gitsha_1) { gitsha(hgsha_1) }
    let(:gitsha_2) { gitsha(hgsha_2) }

    it 'returns commit messages for given commit_ids' do
      messages = described_class.get_messages(
        subject, [hgsha_1, hgsha_0, hgsha_2]
      ).values
      expect(messages).to eq(["Commit car", "Commit foo", "Commit bar"])
      # consistency with HgGitRepository
      messages = described_class.get_messages(
        hg_git_repository, [gitsha_1, gitsha_0, gitsha_2]
      ).values
      expect(messages).to eq(
        ["Commit car\n\n--HG--\nbranch : stable\n",
         "Commit foo\n", "Commit bar\n"]
      )
    end

    it 'returns error if unknown commit_ids sent' do
      expect do
        described_class.get_messages(subject,
                                     [hgsha_1, '134deadbeef', hgsha_2])
      end.to raise_error(GRPC::Internal, /object not found/)
      # consistency with HgGitRepository
      expect do
        described_class.get_messages(hg_git_repository,
                                     [gitsha_1, '134deadbeef', gitsha_2])
      end.to raise_error(GRPC::Internal, /object not found/)
    end
  end

  describe 'commit_service#list_files' do
    subject! { non_empty_repository }

    after do
      subject.remove
    end

    it 'basically works' do
      # we don't need comprehensive tests, just a proof that it works
      # (roundtrip with HGitaly happened, everything was compatible)
      # Full comparison tests with Gitaly are provided in HGitaly.
      expect(subject.ls_files('branch/default')).to eq(["foo"])
    end
  end

  describe 'commit_service#find_commits_by_message' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    def hgsha(rev)
      subject.hgsha_from_rev(rev)
    end

    def gitsha(sha)
      hg_git_repository.hg_git_invalidate_maps!
      hg_git_repository.sha_from_hgsha(sha)
    end

    # repo structure:
    #
    #    o    changeset:   2: Commit bar
    #    |
    #    | o  changeset:   1: Commit car (branch: stable)
    #    |/
    #    o  changeset:   0: Commit 0
    #
    let!(:offset) { 86400 } # seconds in 24 hours
    let(:prepare_repo) do
      hg_commit(subject, 'car', par: '0', branch: 'stable',
                date: "#{offset} 0")
      hg_commit(subject, 'bar', par: '0', date: "#{offset * 2} 0")
    end

    let!(:hgsha_0) { hgsha('0') }
    let!(:hgsha_1) { hgsha('1') }
    let!(:hgsha_2) { hgsha('2') }
    let!(:gitsha_0) { gitsha(hgsha_0) }
    let!(:gitsha_1) { gitsha(hgsha_1) }
    let!(:gitsha_2) { gitsha(hgsha_2) }

    it 'query' do
      commits = subject.gitaly_commit_client.commits_by_message("commit").map(&:id)
      expect(commits).to eq([hgsha_2, hgsha_0])

      commits = subject.gitaly_commit_client.commits_by_message("bar").map(&:id)
      expect(commits).to eq([hgsha_2])

      commits = subject.gitaly_commit_client.commits_by_message("car").map(&:id)
      expect(commits).to eq([])

      # consistency with HgGitRepository
      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "commit"
      ).map(&:id)
      expect(commits).to eq([gitsha_2, gitsha_0])

      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "bar"
      ).map(&:id)
      expect(commits).to eq([gitsha_2])

      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "car"
      ).map(&:id)
      expect(commits).to eq([])
    end

    it 'query+revision' do
      commits = subject.gitaly_commit_client.commits_by_message(
        "car", revision: "branch/stable"
      ).map(&:id)
      expect(commits).to eq([hgsha_1])

      # consistency with HgGitRepository
      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "car", revision: "branch/stable"
      ).map(&:id)
      expect(commits).to eq([gitsha_1])
    end

    it 'query+revision+path' do
      commits = subject.gitaly_commit_client.commits_by_message(
        "commit", path: "foo"
      ).map(&:id)
      expect(commits).to eq([hgsha_0])

      # consistency with HgGitRepository
      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "commit", path: "foo"
      ).map(&:id)
      expect(commits).to eq([gitsha_0])
    end
  end

  describe '#commit_stats' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    # repo structure:
    #
    #    o  changeset:   1 Commit bar
    #    |
    #    o  changeset:   0 Commit 0
    #
    let(:prepare_repo) do
      subject.hg_call(%w[up 0])
      subject.write_file(subject.hg_full_path + "/bar",
                         "I am the first line\n"\
                         "I am the second line\n")
      subject.hg_call(%w[add bar])
      subject.hg_call(['ci', '-m', "Commit bar"],
                      user: user, for_write: true)
      subject.hg_git_invalidate_maps!
    end

    it 'returns commit stats for given revision' do
      hgsha_1 = subject.hgsha_from_rev('1')
      hg_git_repository.hg_git_invalidate_maps!
      gitsha_1 = hg_git_repository.sha_from_hgsha(hgsha_1)

      stats = subject.gitaly_commit_client.commit_stats(hgsha_1)
      expect(stats.additions).to eq(2)
      expect(stats.deletions).to eq(0)
      # consistency with HgGitRepository
      stats = hg_git_repository.gitaly_commit_client.commit_stats(gitsha_1)
      expect(stats.additions).to eq(2)
      expect(stats.deletions).to eq(0)
    end

    it 'returns error if unknown commit_id sent' do
      expect do
        subject.gitaly_commit_client.commit_stats("123notexists321")
      end.to raise_error(GRPC::Internal, /object not found/)
      # consistency with HgGitRepository
      expect do
        hg_git_repository.gitaly_commit_client.commit_stats("123notexists321")
      end.to raise_error(GRPC::Internal, /object not found/)
    end
  end

  describe 'Commit.between' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    # repo structure:
    #
    #    o  changeset:   2 Commit bar2
    #    |
    #    o  changeset:   1 Commit bar
    #    |
    #    o  changeset:   0 Commit 0
    #
    let(:prepare_repo) do
      subject.hg_call(%w[up 0])
      subject.write_file(subject.hg_full_path + "/bar", "content")
      subject.hg_call(['ci', '-Am', "Commit bar"],
                      user: user, for_write: true)
      subject.write_file(subject.hg_full_path + "/bar2", "content")
      subject.hg_call(['ci', '-Am', "Commit bar2"],
                      user: user, for_write: true)
      hg_git_repository.hg_git_invalidate_maps!
    end

    let!(:hg_sha0) { subject.hgsha_from_rev('0') }
    let!(:hg_sha1) { subject.hgsha_from_rev('1') }
    let!(:hg_sha2) { subject.hgsha_from_rev('2') }

    def gitsha(sha)
      hg_git_repository.sha_from_hgsha(sha)
    end

    let(:git_sha0) { gitsha(hg_sha0) }
    let(:git_sha1) { gitsha(hg_sha1) }
    let(:git_sha2) { gitsha(hg_sha2) }

    def between(base, head, repo: subject, **named_args)
      Gitlab::Git::Commit.between(repo, base, head, **named_args)
        .map { |c| c.id }
    end

    it 'returns the expected commits in the appropriate order' do
      # compared to "natural" ascending chronological ordering,
      # this is a double reversal, let's demonstrate with Gitaly first:
      expect(between(git_sha0, git_sha2, repo: hg_git_repository))
        .to eq([git_sha1, git_sha2])
      expect(between(hg_sha0, hg_sha2)).to eq([hg_sha1, hg_sha2])
    end

    it 'associates reverse ordering and limit correctly' do
      # again, non-obvious expected behaviour on the Gitaly side, so let's
      # assert it:
      expect(between(git_sha0, git_sha2, repo: hg_git_repository, limit: 1))
        .to eq([git_sha2])
      expect(between(hg_sha0, hg_sha2, limit: 1)).to eq([hg_sha2])
    end
  end
end

==> heptapod-rails-0.36.2.tar.gz: heptapod-rails-0.36.2/spec/lib/gitlab/mercurial/hgitaly_commit_spec.rb <==
# frozen_string_literal: true

require "spec_helper"

# rubocop:disable RSpec/FilePath
RSpec.describe Gitlab::Git::Commit, :seed_helper do
  include Gitlab::EncodingHelper
  include RepoHelpers
  include Gitlab::Popen

  def hg_commit(repo, file, date: nil, par: nil, branch: nil)
    date = '0 0' if date.nil?
    repo.hg_call(['up', par]) unless par.nil?

    repo.hg_call(['branch', branch.to_s]) unless branch.nil?

    repo.write_file(repo.hg_full_path + "/#{file}",
                       "I am a line in file '#{file}'\n")
    repo.hg_call(['add', file.to_s])
    repo.hg_call(['ci', '-m', "Commit #{file}", '--config',
                  'experimental.single-head-per-branch=False',
                  '-d', date.to_s],
                 user: user, for_write: true)
    repo.hg_git_invalidate_maps!
    repo.hgsha_from_rev('.', repository_path)
  end

  before do
    stub_const('HGITALY_TEST_REPO_PATH', 'hgitaly-repo.git')
  end

  let(:repository) do
    repo = Gitlab::Mercurial::HgitalyRepository.new('default', HGITALY_TEST_REPO_PATH, '', 'group/project')
    repo.unit_tests_skip_hooks = true
    repo
  end

  # Same repository as :repository, seen as HgGitRepository, so that
  #
  #   1. we can use HgGitRepository as a reference for assertions on method
  #      results
  #   2. we have easy fixture preparation options
  let(:hg_git_repository) do
    repo = Gitlab::Mercurial::HgGitRepository.new('default', HGITALY_TEST_REPO_PATH, '', 'group/project')
    repo.unit_tests_skip_hooks = true
    repo
  end

  let(:repository_path) { "#{File.join(TestEnv.repos_path, repository.relative_path).chomp('.git')}.hg" }
  let(:user) { build(:user) }
  let(:non_empty_repository) do
    status = hg_git_repository.create_repository(namespace_fs_path: "group",
                                                 in_subgroup: false)
    expect(status).to be(true)
    hg_commit(hg_git_repository, 'foo', date: "0 0")
    repository # HgitalyRepository
  end

  let(:non_empty_repository_path) { repository_path }

  describe '#find_all' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    def hgsha(rev)
      subject.hgsha_from_rev(rev)
    end

    def gitsha(sha)
      hg_git_repository.hg_git_invalidate_maps!
      hg_git_repository.sha_from_hgsha(sha)
    end

    # repo structure:
    #
    #    o    changeset:   2: Commit bar
    #    |
    #    | o  changeset:   1: Commit car (branch: stable)
    #    |/
    #    o  changeset:   0: Commit foo
    #
    let!(:offset) { 86400 } # seconds in 24 hours
    let(:prepare_repo) do
      hg_commit(subject, 'car', par: '0', branch: 'stable',
                                date: "#{offset} 0")
      hg_commit(subject, 'bar', par: '0', date: "#{offset * 2} 0")
    end

    let(:hgsha_0) { hgsha('0') }
    let(:hgsha_1) { hgsha('1') }
    let(:hgsha_2) { hgsha('2') }
    let(:gitsha_0) { gitsha(hgsha_0) }
    let(:gitsha_1) { gitsha(hgsha_1) }
    let(:gitsha_2) { gitsha(hgsha_2) }

    it 'returns a collection of commits' do
      commits = described_class.find_all(subject).map(&:id)
      expect(commits).to eq([hgsha_2, hgsha_1, hgsha_0])
      # consistency with HgGitRepository
      commits = described_class.find_all(hg_git_repository).map(&:id)
      expect(commits).to eq([gitsha_2, gitsha_1, gitsha_0])
    end

    context 'max_count' do
      it 'return maximum max_count number of commits' do
        commits = described_class.find_all(subject, max_count: 2).map(&:id)
        expect(commits).to eq([hgsha_2, hgsha_1])
        # consistency with HgGitRepository
        commits = described_class.find_all(hg_git_repository, max_count: 2).map(&:id)
        expect(commits).to eq([gitsha_2, gitsha_1])
      end
    end

    context 'ref + max_count + skip' do
      it 'returns a collection of commits' do
        commits = described_class.find_all(
          subject,
          ref: 'branch/default',
          max_count: 1,
          skip: 1
        ).map(&:id)
        expect(commits).to eq([hgsha_0])
        # consistency with HgGitRepository
        commits = described_class.find_all(
          hg_git_repository,
          ref: 'branch/default',
          max_count: 1,
          skip: 1
        ).map(&:id)
        expect(commits).to eq([gitsha_0])
      end
    end
  end

  describe '#get_messages' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    def hgsha(rev)
      subject.hgsha_from_rev(rev)
    end

    def gitsha(sha)
      hg_git_repository.hg_git_invalidate_maps!
      hg_git_repository.sha_from_hgsha(sha)
    end

    # repo structure:
    #
    #    o    changeset:   2: Commit bar
    #    |
    #    | o  changeset:   1: Commit car (branch: stable)
    #    |/
    #    o  changeset:   0: Commit foo
    #
    let!(:offset) { 86400 } # seconds in 24 hours
    let(:prepare_repo) do
      hg_commit(subject, 'car', par: '0', branch: 'stable',
                                date: "#{offset} 0")
      hg_commit(subject, 'bar', par: '0', date: "#{offset * 2} 0")
    end

    let(:hgsha_0) { hgsha('0') }
    let(:hgsha_1) { hgsha('1') }
    let(:hgsha_2) { hgsha('2') }
    let(:gitsha_0) { gitsha(hgsha_0) }
    let(:gitsha_1) { gitsha(hgsha_1) }
    let(:gitsha_2) { gitsha(hgsha_2) }

    it 'returns commit messages for given commit_ids' do
      messages = described_class.get_messages(
        subject, [hgsha_1, hgsha_0, hgsha_2]
      ).values
      expect(messages).to eq(["Commit car", "Commit foo", "Commit bar"])
      # consistency with HgGitRepository
      messages = described_class.get_messages(
        hg_git_repository, [gitsha_1, gitsha_0, gitsha_2]
      ).values
      expect(messages).to eq(
        ["Commit car\n\n--HG--\nbranch : stable\n",
         "Commit foo\n", "Commit bar\n"]
      )
    end

    it 'returns error if unknown commit_ids sent' do
      expect do
        described_class.get_messages(subject,
                                     [hgsha_1, '134deadbeef', hgsha_2])
      end.to raise_error(GRPC::Internal, /object not found/)
      # consistency with HgGitRepository
      expect do
        described_class.get_messages(hg_git_repository,
                                     [gitsha_1, '134deadbeef', gitsha_2])
      end.to raise_error(GRPC::Internal, /object not found/)
    end
  end

  describe 'commit_service#list_files' do
    subject! { non_empty_repository }

    after do
      subject.remove
    end

    it 'basically works' do
      # we don't need comprehensive tests, just a proof that it works
      # (roundtrip with HGitaly happened, everything was compatible)
      # Full comparison tests with Gitaly are provided in HGitaly.
      expect(subject.ls_files('branch/default')).to eq(["foo"])
    end
  end

  describe 'commit_service#find_commits_by_message' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    def hgsha(rev)
      subject.hgsha_from_rev(rev)
    end

    def gitsha(sha)
      hg_git_repository.hg_git_invalidate_maps!
      hg_git_repository.sha_from_hgsha(sha)
    end

    # repo structure:
    #
    #    o    changeset:   2: Commit bar
    #    |
    #    | o  changeset:   1: Commit car (branch: stable)
    #    |/
    #    o  changeset:   0: Commit 0
    #
    let!(:offset) { 86400 } # seconds in 24 hours
    let(:prepare_repo) do
      hg_commit(subject, 'car', par: '0', branch: 'stable',
                                date: "#{offset} 0")
      hg_commit(subject, 'bar', par: '0', date: "#{offset * 2} 0")
    end

    let!(:hgsha_0) { hgsha('0') }
    let!(:hgsha_1) { hgsha('1') }
    let!(:hgsha_2) { hgsha('2') }
    let!(:gitsha_0) { gitsha(hgsha_0) }
    let!(:gitsha_1) { gitsha(hgsha_1) }
    let!(:gitsha_2) { gitsha(hgsha_2) }

    it 'query' do
      commits = subject.gitaly_commit_client.commits_by_message("commit").map(&:id)
      expect(commits).to eq([hgsha_2, hgsha_0])

      commits = subject.gitaly_commit_client.commits_by_message("bar").map(&:id)
      expect(commits).to eq([hgsha_2])

      commits = subject.gitaly_commit_client.commits_by_message("car").map(&:id)
      expect(commits).to eq([])

      # consistency with HgGitRepository
      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "commit"
      ).map(&:id)
      expect(commits).to eq([gitsha_2, gitsha_0])

      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "bar"
      ).map(&:id)
      expect(commits).to eq([gitsha_2])

      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "car"
      ).map(&:id)
      expect(commits).to eq([])
    end

    it 'query+revision' do
      commits = subject.gitaly_commit_client.commits_by_message(
        "car", revision: "branch/stable"
      ).map(&:id)
      expect(commits).to eq([hgsha_1])

      # consistency with HgGitRepository
      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "car", revision: "branch/stable"
      ).map(&:id)
      expect(commits).to eq([gitsha_1])
    end

    it 'query+revision+path' do
      commits = subject.gitaly_commit_client.commits_by_message(
        "commit", path: "foo"
      ).map(&:id)
      expect(commits).to eq([hgsha_0])

      # consistency with HgGitRepository
      commits = hg_git_repository.gitaly_commit_client.commits_by_message(
        "commit", path: "foo"
      ).map(&:id)
      expect(commits).to eq([gitsha_0])
    end
  end

  describe '#commit_stats' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    # repo structure:
    #
    #    o  changeset:   1 Commit bar
    #    |
    #    o  changeset:   0 Commit 0
    #
    let(:prepare_repo) do
      subject.hg_call(%w[up 0])
      subject.write_file("#{subject.hg_full_path}/bar",
                         "I am the first line\n"\
                         "I am the second line\n")
      subject.hg_call(%w[add bar])
      subject.hg_call(['ci', '-m', "Commit bar"],
                      user: user, for_write: true)
      subject.hg_git_invalidate_maps!
    end

    it 'returns commit stats for given revision' do
      hgsha_1 = subject.hgsha_from_rev('1')
      hg_git_repository.hg_git_invalidate_maps!
      gitsha_1 = hg_git_repository.sha_from_hgsha(hgsha_1)

      stats = subject.gitaly_commit_client.commit_stats(hgsha_1)
      expect(stats.additions).to eq(2)
      expect(stats.deletions).to eq(0)
      # consistency with HgGitRepository
      stats = hg_git_repository.gitaly_commit_client.commit_stats(gitsha_1)
      expect(stats.additions).to eq(2)
      expect(stats.deletions).to eq(0)
    end

    it 'returns error if unknown commit_id sent' do
      expect do
        subject.gitaly_commit_client.commit_stats("123notexists321")
      end.to raise_error(GRPC::Internal, /object not found/)
      # consistency with HgGitRepository
      expect do
        hg_git_repository.gitaly_commit_client.commit_stats("123notexists321")
      end.to raise_error(GRPC::Internal, /object not found/)
    end
  end

  describe 'Commit.between' do
    subject! { non_empty_repository }

    before do
      prepare_repo
    end

    after do
      subject.remove
    end

    # repo structure:
    #
    #    o  changeset:   2 Commit bar2
    #    |
    #    o  changeset:   1 Commit bar
    #    |
    #    o  changeset:   0 Commit 0
    #
    let(:prepare_repo) do
      subject.hg_call(%w[up 0])
      repo_path = subject.hg_full_path
      subject.write_file("#{repo_path}/bar", "content")
      subject.hg_call(['ci', '-Am', "Commit bar"],
                      user: user, for_write: true)
      subject.write_file("#{repo_path}/bar2", "content")
      subject.hg_call(['ci', '-Am', "Commit bar2"],
                      user: user, for_write: true)
      hg_git_repository.hg_git_invalidate_maps!
    end

    let(:git_sha0) { gitsha(hg_sha0) }
    let(:git_sha1) { gitsha(hg_sha1) }
    let(:git_sha2) { gitsha(hg_sha2) }

    let!(:hg_sha0) { subject.hgsha_from_rev('0') }
    let!(:hg_sha1) { subject.hgsha_from_rev('1') }
    let!(:hg_sha2) { subject.hgsha_from_rev('2') }

    def gitsha(sha)
      hg_git_repository.sha_from_hgsha(sha)
    end

    def between(base, head, repo: subject, **named_args)
      Gitlab::Git::Commit.between(repo, base, head, **named_args)
        .map(&:id)
    end

    it 'returns the expected commits in the appropriate order' do
      # compared to "natural" ascending chronological ordering,
      # this is a double reversal, let's demonstrate with Gitaly first:
      expect(between(git_sha0, git_sha2, repo: hg_git_repository))
        .to eq([git_sha1, git_sha2])
      expect(between(hg_sha0, hg_sha2)).to eq([hg_sha1, hg_sha2])
    end

    it 'associates reverse ordering and limit correctly' do
      # again, non-obvious expected behaviour on the Gitaly side, so let's
      # assert it:
      expect(between(git_sha0, git_sha2, repo: hg_git_repository, limit: 1))
        .to eq([git_sha2])
      expect(between(hg_sha0, hg_sha2, limit: 1)).to eq([hg_sha2])
    end
  end
end
# rubocop:enable RSpec/FilePath
